from images import Image

# --------------------------------
# functions of pixel -> number
# --------------------------------

def average_intensity(r, g, b):
    '''Uses simple average.'''
    return (r + g + b) // 3

def optimized_gray(r, g, b):
    '''Uses ratios optimized for human vision.'''
    r = int(r * 0.299)
    g = int(g * 0.587)
    b = int(b * 0.114)
    return r + g + b

# --------------------------------
# convert image to grayscale
# --------------------------------

gray_for = optimized_gray

def grayscale(image):
    '''Converts the argument image to grayscale.'''
    for x in range(image.getWidth()):
        for y in range(image.getHeight()):
            r, g, b = image.getPixel(x, y)
            value = gray_for(r, g, b)
            image.setPixel(x, y, (value, value, value))

# --------------------------------
# remove all blue from an image
# --------------------------------

def clear_blue(image):
    '''Sets the blue value of every pixel to 0.'''
    for x in range(image.getWidth()):
        for y in range(image.getHeight()):
            r, g, b = image.getPixel(x, y)
            image.setPixel(x, y, (r, g, 0))

# --------------------------------
# create a photo negative
# --------------------------------

def negate(image):
    '''Create a reverse image, like a photo negative.'''
    for x in range(image.getWidth()):
        for y in range(image.getHeight()):
            r, g, b = image.getPixel(x, y)
            image.setPixel(x, y, (255-r, 255-g, 255-b))

# ----------------------------------------
# change all pixels to red, blue, or green
# ----------------------------------------

def three_colors(image):
    '''A really simple posterizing effect.'''
    for x in range(image.getWidth()):
        for y in range(image.getHeight()):
            r, g, b = image.getPixel(x, y)
            average = average_intensity(r, g, b)
            if average < 85:
                image.setPixel(x, y, (0, 255, 0))
            elif average < 170:
                image.setPixel(x, y, (255, 0, 0))
            else:
                image.setPixel(x, y, (0, 0, 255))

# -------------------------------------
# in honor of my grad school alma mater
# -------------------------------------

def purple_to_green(image):
    '''Just for the Cotton Bowl, change my purple shirt to Kelly green.'''
    threshold = 40
    target_r, target_g, target_b = (163, 124, 143)
    for x in range(image.getWidth()):
        for y in range(image.getHeight()):
            r, g, b = image.getPixel(x, y)
            if abs(r - target_r) < threshold and \
               abs(g - target_g) < threshold and \
               abs(b - target_b) < threshold:
                r = r // 2
                b = b // 2
                g = g - (g - 124) // 5
                image.setPixel(x, y, (r, g, b))

# -------------------------------------
# find 'edges' in in an image
# -------------------------------------

def detect_edges(image):
    '''Convert image into a line drawing at sharp changes.'''
    threshold = 10
    for x in range(image.getWidth() - 1):
        for y in range(image.getHeight()):
            left_r,  left_g,  left_b  = image.getPixel(x, y)
            right_r, right_g, right_b = image.getPixel(x+1, y)

            left_average  = average_intensity(left_r, left_g, left_b)
            right_average = average_intensity(right_r, right_g, right_b)
            difference    = abs(left_average - right_average)
            
            if difference > threshold:
                image.setPixel(x, y, (0, 0, 0))
            else:
                image.setPixel(x, y, (255, 255, 255))

# -------------------------------------
# 'posterize' an image
# -------------------------------------

def force_value(n):
    if n < 64:
        return 31
    elif n < 128:
        return 95
    elif n < 192:
        return 159
    else:
        return 223

def posterize(image):
    '''Change every color into one of four possible values.'''
    threshold = 10
    for x in range(image.getWidth() - 1):
        for y in range(image.getHeight()):
            r, g, b = image.getPixel(x, y)
            r = force_value(r)
            b = force_value(b)
            g = force_value(g)
            image.setPixel(x, y, (r, g, b))

# -------------------------------------
# make image look hazy, as if in fog
# -------------------------------------

##def fog_effect(image):
##    '''Convert image into a line drawing at sharp changes.'''
##    threshold = 200
##    for x in range(image.getWidth()):
##        for y in range(image.getHeight()):
##            r, g, b = image.getPixel(x, y)
##            if r > threshold:
##                image.setPixel(x, y, (b, b, b))

# --------------------------------------------------------------------------
# main method for applying a filter to an image
# --------------------------------------------------------------------------

def apply_filter(img_filter, filename = 'smokey.gif'):
    image = Image(filename)
    print('Close the image window to continue.')
    image.draw()
    img_filter(image)
    print('Close the image window to quit.')
    image.draw()

apply_filter(three_colors, 'wallingf-umpr2006.gif')
